function [Phi_S, C, P] = HOSLOC(G, v, phi, t_max, b, xi, motif)
% Input:
% - G: the adjacency matrix of the given network.
% - v: the seed vertex.
% - phi: the user-defined conductance upper-bound of the desired graph cut. 
% - t_max: the maximum iteration number. 
% - b: the parameter controls the size of the returned cluster.
% - xi: the parameter regarding the diffusion core.
% - motif: the user-defined structures including 'ThreeNodeLine', 'Triangle', 'FourNodeLoop', 'FiveNodeStar'.
% Output:
% - Phi_S: the conductance of the returned cluster. 
% - C: the returned structure-rich graph cut. 
% - P: the constructed transition tensor. 

%% Parameter settings
G = sparse(G);
n = size(G,1);
c1=140;
mu_V = full(sum(sum(G)));
l = log2(mu_V)/2;
epi = 1/(1800*(l+2)*t_max*2^b);
d = sum(G,2);

%% Initialization
I=speye(n);
inv_D = 1./d;
M=(G*sparse(diag(inv_D))+I)/2; 

S=sprintf('------Build Transition Tensor------');
disp(S)
if strcmp(motif, 'ThreeNodeLine')
    P = TP_ThreeNodeLine(G);
    vec = sum(P,1);
    ind = find(vec);
    vec(ind) = 1./vec(ind);
    D_P = diag(vec);
    P = P*D_P;
    q_1=sparse(n,1);
    q_1(v)=1;
    q_2 = M*q_1;
    r_1 = vec_epi(G,q_1,epi);
    r_2 = vec_epi(G,q_2,epi);
    C = [];
end

if strcmp(motif, 'Triangle')
    P = TP_Triangle(G);
    vec = sum(P,1);
    ind = find(vec);
    vec(ind) = 1./vec(ind);
    D_P = diag(vec);
    P = P*D_P;
    q_1=sparse(n,1);
    q_1(v)=1;
    q_2 = M*q_1;
    r_1 = vec_epi(G,q_1,epi);
    r_2 = vec_epi(G,q_2,epi);
    C = [];
end

if strcmp(motif, 'FourNodeLoop')
    P = TP_FourNodeLoop(G);
    vec = sum(P,1);
    ind = find(vec);
    vec(ind) = 1./vec(ind);
    D_P = diag(vec);
    P = P*D_P;
    q_1=sparse(n,1);
    q_1(v)=1;
    q_2 = M*q_1;
    q_3 = M*q_2;
    r_1 = vec_epi(G,q_1,epi);
    r_2 = vec_epi(G,q_2,epi);
    r_3 = vec_epi(G,q_3,epi);
    C = [];
end

if strcmp(motif, 'FiveNodeStar')
    P = TP_FiveNodeStar(G);
    vec = sum(P,1);
    ind = find(vec);
    vec(ind) = 1./vec(ind);
    D_P = diag(vec);
    P = P*D_P;
    q_1=sparse(n,1);
    q_1(v)=1;
    q_2 = M*q_1;
    q_3 = M*q_2;
    q_4 = M*q_3;
    r_1 = vec_epi(G,q_1,epi);
    r_2 = vec_epi(G,q_2,epi);
    r_3 = vec_epi(G,q_3,epi);
    r_4 = vec_epi(G,q_4,epi);
    C = [];
end

%% Main
S=sprintf('------Clustering------');
disp(S)

for t = 1:t_max
    if strcmp(motif, 'ThreeNodeLine')
        q_1 = P*kron(r_1',r_2');
        q_1 = q_1/sum(q_1);
        r_1 = r_2;
        r_2 = vec_epi(G,q_1,epi);
    end
    
    if strcmp(motif, 'Triangle')
        q_1 = P*kron(r_1',r_2');
        q_1 = q_1/sum(q_1);
        r_1 = r_2;
        r_2 = vec_epi(G,q_1,epi);
    end
    
    if strcmp(motif, 'FourNodeLoop')
        q_1 = P*kron(kron(r_1',r_2'),r_3');
        q_1 = q_1/sum(q_1);
        r_1 = r_2;
        r_2 = r_3;
        r_3 = vec_epi(G,q_1,epi);
    end
   
    if strcmp(motif, 'FiveNodeStar')
        q_1 = P*kron(kron(kron(r_1',r_2'),r_3'),r_4');
        q_1 = q_1/sum(q_1);
        r_1 = r_2;
        r_2 = r_3;
        r_3 = r_4;
        r_4 = vec_epi(G,q_1,epi);
    end
    I_s = q_1./d;
    [B, I] = sort(I_s,'descend');
    for j = 1:n  
        index = I(1:j);
        temp = sum(sum(G(index,index)));
        namta = full(sum(d(index)));
        if namta - temp==0
            Phi_S=0;
        else
            Phi_S = (namta- temp)/ min(namta, mu_V - namta);
        end
        I_x = full(B(j));    
        if Phi_S <= phi && namta <= 5/6*mu_V && 2^b <= namta && I_x >= xi/(c1*(l+2)*2^b)
            C = index';
            return
        else
            continue
        end
    end
end





